/*****************************************************************************
 **
 ** Licensed Materials - Property of IBM
 **
 ** (C) COPYRIGHT International Business Machines Corp. 2000, 2001
 ** All Rights Reserved.
 **
 ** DB2 Everyplace Demo Engine                                     
 **                                                                         
 ** C++ interface to the DB2 Everyplace database.                           
 **
 ** The DB2 Everyplace Demo engine is an intermediate layer between the
 ** Visiting Nurse application and the DB2 Everyplace database engine with
 ** the purpose to hide all the environment, connection and statment handles          
 ** from the specific design of the application.                   
 **                                                                         
 ** This piece of code can be reused for every other application   
 **                                                                         
 ***************************************************************************
 **                              DB2eEng.h                                  
 ***************************************************************************/

#ifndef __DB2EENG_H__
#define __DB2EENG_H__

/*****************************************************************************/
/**  Include files                                                          **/
/*****************************************************************************/

#include <coemain.h>
#include <stdlib.h>  // include the C-stdlib for CloseSTDLIB()
#include "sqlcli.h"

#if defined(UNICODE) || defined(_UNICODE)
  #define CHARTYPE unsigned short
#else
  #define CHARTYPE char
#endif

#ifndef _TEXT
  #if defined(UNICODE) || defined(_UNICODE)
    #define _TEXT(s) ((SQLTCHAR*)L ## s)
  #else
    #define _TEXT(s) ((SQLTCHAR*)s)
  #endif
#endif

/*****************************************************************************/
/**  Type definitions                                                       **/
/*****************************************************************************/

/** definition of the return codes of the DB2 Everyplace demo engine
  */

enum DB2eRETURN
  {
    SUCCESS = 0,  // everything went fine
    FAILURE = 1,  // any error accured
    NODATA  = 2   // no data to fetch anymore
  };

#define DB2eERROR_LOCATION_LENGTH 100
#define DB2eERROR_MESSAGE_LENGTH  100
#define DB2eERROR_STATE_LENGTH    10
#define DB2e_MESSAGE_LENGTH       200

/** type definition for storing the informations of multiple database
  *   connections (not implemented in this demo yet!)
  */

typedef struct sdbcstmt
{
  SQLHDBC          hdbc;      // database connection handle
  SQLHSTMT        hstmt;      // statement handle
} DB2eHdbcStmt;

/** type definition for storing the informations of
  *   binded parameters list
  */

typedef struct sbindParam
{
  short number;           // parameter number
  short ctype;            // C data type of the parameter
  short type;             // SQL data type of the parameter
  short columnSize;       // maximum length in bytes for this parameter
  short decimalDigits;    // number of decimal digits
  void* pValue;           // points to the buffer which contains the actual 
                          // data of the parameter
  int   buffLength;       // length of the buffer
  long* length;           // contains the length of the patametermarker stored
                          //   at pValue

  struct sbindParam* next;// pointer to the next member of the bind parameter
                          //   list. (NULL == end of list)
} DB2eBindParam;



/*****************************************************************************/
/**  Class definitions                                                      **/
/*****************************************************************************/

/** The DB2 Everyplace Demo engine is an intermediate layer to hide
  *    all the environment-, connection and statment handles
  *    from the specific design of the application
  */

class CDB2eEngine : public CBase
{
  public:
    // starts the DB2 Everyplace engine
    DB2eRETURN  DB2eStartEngine       ();    

    // stops the DB2 Everyplace engine
    void        DB2eStopEngine        ();     

    // executes an SQL statement 
    DB2eRETURN  DB2eExecStatement     (CHARTYPE* sqlstmt);

    // executes an SQl statement with binding the parameter list
    DB2eRETURN  DB2eExecBindStatement (CHARTYPE* sqlstmt, 
                                       DB2eBindParam* param);

    
    // fetches the data
    DB2eRETURN  DB2eFetchData         ();

    // get CHAR, DECIMAL, DATE, TIME, VARCHAR data
    DB2eRETURN  DB2eGetData           (int       colNum, 
                                       CHARTYPE* val, 
                                       long      size, 
                                       long*     length);

    // get SMALLINT data
    DB2eRETURN  DB2eGetData           (int    colNum, 
                                       short* val, 
                                       long*  length);

    // get INTEGER data
    DB2eRETURN  DB2eGetData           (int   colNum, 
                                       long* val, 
                                       long* length);

    // get error message
    TPtrC        DB2eGetErrorMsg      ();
    // get error location
    TPtrC        DB2eGetErrorLocation ();
    // get error state
    TPtrC        DB2eGetErrorState    ();


  private:
    // build error report
    void      DB2eErrorReport    (SQLHSTMT    hstmt,
                                  TPtrC       location,
                                  TPtrC       place);

    SQLHENV   getHEnv    ();               // get environment handle
    SQLHDBC   getHDbc    ();               // get database connection
    SQLHSTMT  getHStmt   ();               // get statement handle
    void      setHEnv    (SQLHENV  henv);  // set environment handle
    void      setHDbc    (SQLHDBC  hdbc);  // set database connection
    void      setHStmt   (SQLHSTMT hstmt); // set statement handle

    SQLHENV        iHenv;          // environment handle
    SQLHDBC        iHdbc;          // database connection handle
    SQLHSTMT      iHstmt;          // statement handle
    TBuf<DB2eERROR_LOCATION_LENGTH> iErrorLocation; // error location information 
    TBuf<DB2eERROR_MESSAGE_LENGTH>  iErrorMessage;  // error message
    TBuf<DB2eERROR_STATE_LENGTH>    iErrorState;    // error state
};
 
#endif
