/*******************************************************************************
 **
 ** Licensed Materials - Property of IBM
 **
 ** (C) COPYRIGHT International Business Machines Corp. 2000, 2001
 ** All Rights Reserved.
 **
 ******************************************************************************/
#include "CLPview.h"
#include "CLPappui.h"

_LIT(KTxtCDrive,"C:");
_LIT(KTxtZDrive,"Z:");

CCLPView::~CCLPView()
{    
  if(iCLPText)
    delete iCLPText;

  if(iCLPQuery)
    delete iCLPQuery;

  if(iCLPQueryHint)
    delete iCLPQueryHint;

  if(iCLPResults)
    delete iCLPResults;

  if(iMSG_CLP_MSG_STMT_RST)
    delete iMSG_CLP_MSG_STMT_RST;

  if(iTxLstBx)
  {
     CTextListBoxModel* aModel = iTxLstBx->Model();
     MDesCArray* itemList  = aModel->ItemTextArray();
     CDesCArray* itemArray = (CDesCArray*)itemList;
     if (itemArray) {
       aModel->SetOwnershipType(ELbmDoesNotOwnItemArray);
       itemArray->Reset();
       aModel->SetOwnershipType(ELbmOwnsItemArray);
     }
     delete iTxLstBx;
  }

  if(iCombo[3])
  {
    CDesCArray* itemArray = (CDesCArray*)(iCombo[3]->Array());
    if(itemArray) {
      itemArray->Reset();
      delete itemArray;
    }
    delete iCombo[3];
  }

  if(iFont)
    iCoeEnv->ReleaseScreenFont(CONST_CAST(CFont*,iFont));
}

void CCLPView::ConstructL(const TRect& aRect,
                          CLPState*    clpstate,
                          CCLPAppUi*   aAppUi)
{
  iAppUi = aAppUi;
  iclpstate = clpstate;

  // Create Window and attributes
  CreateWindowL();
  Window().SetShadowDisabled(ETrue);
  SetBlank();
  Window().SetBackgroundColor(KRgb1in4DitheredGray); // light gray
  SetRectL(aRect);
  
   //initialize pointer
  iFont = 0;
  iTxLstBx = 0;
  iCombo[3] = 0;

  // set font for the title
  TFontSpec spec(_L("Arial"), 140);
  /************************************************************************** 
  Font: Arial, Times, Courier, Symbol, Calc, Digital, Calcinv, Comms, Eikon
  TFontSpec - TInt iHeight, iTypeface, iFontStyle;
  enum TFontPosture - EPostureUpright, EPostureItalic
  enum TFontPrintPosition - EPrintPosNormal,EPrintPosSuperscript,EPrintPosSubscript
  enum TFontStrokeWeight - EStrokeWeightBold, EStrokeWeightNormal
  **************************************************************************/  
  spec.iFontStyle.SetStrokeWeight(EStrokeWeightBold);
  SetFontL(spec);

  // Title line of ClpView 
  iCLPText=iCoeEnv->AllocReadResourceL(CLP_VIEW_TITLE); 

  iMSG_CLP_MSG_STMT_RST = iCoeEnv->AllocReadResourceL(MSG_CLP_MSG_STMT_RST);

  // Set font
  const CFont* afont = iEikonEnv->NormalFont();

  // Create Lables: Query, Hint, Results
  iCLPQuery=iCoeEnv->AllocReadResourceL(CLP_VIEW_QUERY);
  iCLPQueryHint=iCoeEnv->AllocReadResourceL(CLP_VIEW_QUERY_HINT);
  iCLPResults=iCoeEnv->AllocReadResourceL(CLP_VIEW_RESULTS);  
  
  // Create Combo Box
  CreateCombosL();
  iCombo[3]->SetFocus(ETrue);

  // Create List Box
  CreateTxLstBoxL(aRect,afont);
  
  // create icons in CLP View
  CreateBitMapsL();

  ActivateL();
}

void CCLPView::SetFontL(const TFontSpec& aFontSpec)
{
  if(iFont)
    iCoeEnv->ReleaseScreenFont(CONST_CAST(CFont*,iFont)); // after previous line succeeds
  iFont = iCoeEnv->CreateScreenFontL(aFontSpec);
  iFontSpec=aFontSpec;
}

void CCLPView::CreateBitMapsL()
{
  TBool shareIfLoaded(ETrue);
}

TInt CCLPView::CountComponentControls() const
{
  return 2;
}

CCoeControl* CCLPView::ComponentControl(TInt aIndex) const
{
  CCoeControl* aControl = NULL;

  switch (aIndex)
  {
    case 0:
      aControl = iCombo[aIndex+3];
      break;

    case 1:
      aControl = iTxLstBx;
      break;

    default:
      aControl = NULL;
  }

  return aControl;
}

void CCLPView::CreateCombosL()
{
  #define qPREFS_MAX_BUFFER 1
  iCombo[3]=new(ELeave) CEikComboBox;
  iCombo[3]->ConstructL(*this,10,0,qPREFS_MAX_BUFFER);
  iCombo[3]->SetArray((new(ELeave) CDesCArrayFlat(qPREFS_MAX_BUFFER)),
                      CEikComboBox::EArrayExternalOwner);
  CDesCArray* itemArray = (CDesCArray*)(iCombo[3]->Array());

  for (int i = 0; i < qPREFS_MAX_BUFFER; i++)
    itemArray->AppendL(_L(" "));
  
  TSize size=iCombo[3]->MinimumSize();
  size.iHeight*=1;  // one line
  size.iWidth+=311;
  iCombo[3]->SetExtentL(TPoint(69,29),size);
  iCombo[3]->SetObserver(this);
  iCombo[3]->SetFocus(ETrue);
}

void CCLPView::ExecuteQueryL()
{
  #define DB2e_MAX_SQL_STMT_SIZE 2048
  
  int rc = 0;
  SQLTCHAR* sqlstring;
  CEikComboBox* combo = (iCombo[3]);
  TBuf<DB2e_MAX_SQL_STMT_SIZE> buf;

  sqlstring = (SQLTCHAR*) malloc (sizeof(SQLTCHAR) * DB2e_MAX_SQL_STMT_SIZE);
  
  memset(sqlstring, 0, sizeof(SQLTCHAR) * DB2e_MAX_SQL_STMT_SIZE);
       
  combo->GetText(KsqlString);
  // convert string
  memcpy(sqlstring, KsqlString.PtrZ(), KsqlString.Length()+1);

  for(int i = 0; i < KsqlString.Length(); i++)
    if (sqlstring[i] == 16)
      sqlstring[i] = ' ';

   memset(iclpstate->cmd, 0, MAX_CMD_LEN);
   memcpy(iclpstate->cmd, sqlstring, KsqlString.Length());
   rc = CLPcommand_interpreter(iclpstate);
  
exit:
  if (sqlstring)
    free(sqlstring); 
}

void CCLPView::PrintMsg(SQLTCHAR* msg)
{
  TBuf<1000> buf;
  buf.Format(*iMSG_CLP_MSG_STMT_RST, msg);
  AddToListBoxL(buf); 
}

TKeyResponse CCLPView::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
  CEikComboBox* focused=(iCombo[3]); 

  if (aType!=EEventKey)
    return(EKeyWasNotConsumed);


  if (aKeyEvent.iModifiers&EModifierCtrl)
  {
    switch(aKeyEvent.iCode/*code*/)
    {
      case CTRL('e'):
        CBaActiveScheduler::Exit();
       
      default:
        focused->OfferKeyEventL(aKeyEvent,aType);
     }
  }
  else
  {
    switch (aKeyEvent.iCode/*code*/)
    {
      case EKeyEnter:
      {
        ExecuteQueryL();

        iTxLstBx->HandleItemAdditionL();
        iTxLstBx->ActivateL();
        iTxLstBx->DrawNow();
        break;
      }

      case EKeyEscape:
      {
        CDesCArray* desCArray=((CDesCArray*)focused->Array());
        if (desCArray->Count()>0)
          focused->SetTextL(&((*desCArray)[0]));
        else
        {
          TPtrC ptr;
          focused->SetTextL(&ptr);
        }
        break;
      }

      default:
        focused->OfferKeyEventL(aKeyEvent,aType);
    }
  }

  return(EKeyWasConsumed);
}

void CCLPView::HandleCommandL(TInt aCommand)
{  
  TRect aRect;
  aRect.SetHeight(240);
  aRect.SetWidth(570);

  switch (aCommand)
  {
    case CmdClpClear:
      if (!iAppUi->iIsSplashView)
      {
        if (iTxLstBx)
        {
          CTextListBoxModel* aModel = iTxLstBx->Model();
          MDesCArray* itemList  = aModel->ItemTextArray();
          CDesCArray* itemArray = (CDesCArray*)itemList;
          if (itemArray)
          {
            aModel->SetOwnershipType(ELbmDoesNotOwnItemArray);
            itemArray->Reset();
            aModel->SetOwnershipType(ELbmOwnsItemArray);
          }
          delete iTxLstBx;
        }   
        const CFont* afont = iEikonEnv->NormalFont();
        CreateTxLstBoxL(aRect,afont);
        DrawNow();
      }
      break;

    case CmdRunSql:
      if (!iAppUi->iIsSplashView)
      {
        ExecuteQueryL();
        iTxLstBx->HandleItemAdditionL();
        iTxLstBx->ActivateL();
        iTxLstBx->DrawNow();
      }
      break;
  }
}

void CCLPView::HandleControlEventL(CCoeControl* aControl,TCoeEvent) {}

void CCLPView::AddToListBoxL(const TDesC& aText)
{  
  MDesCArray* itemList  = iTxLstBx->Model()->ItemTextArray();
  CDesCArray* itemArray = (CDesCArray*)itemList;
  itemArray->AppendL(aText);
}

void CCLPView::CreateTxLstBoxL(const TRect& aRect,const CFont* iFont)
{
  // create schedule listbox
  iTxLstBx = new(ELeave) CEikTextListBox;
  iTxLstBx->ConstructL(this, 0 );

  // create scrollbar
  iTxLstBx->CreateScrollBarFrameL();
  iTxLstBx->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto);
  
  // set size
  iTxLstBx->SetExtentL(TPoint(10, 58), TSize(aRect.Width() - 16, aRect.Height() - 66));
  iTxLstBx->HandleItemAdditionL();
  
  // set font for the title
  TFontSpec spec(_L("Arial"), 140);
  /************************************************************************** 
  Font: Arial, Times, Courier, Symbol, Calc, Digital, Calcinv, Comms, Eikon
  TFontSpec - TInt iHeight, iTypeface, iFontStyle;
  enum TFontPosture - EPostureUpright, EPostureItalic
  enum TFontPrintPosition - EPrintPosNormal,EPrintPosSuperscript,EPrintPosSubscript
  enum TFontStrokeWeight - EStrokeWeightBold, EStrokeWeightNormal
  **************************************************************************/  
  spec.iFontStyle.SetStrokeWeight(EStrokeWeightBold);
  this->SetFontL(spec); 

  iTxLstBx->ActivateL();
  iTxLstBx->DrawNow();
}

void CCLPView::Draw(const TRect& aRect) const
{
  CWindowGc& gc = SystemGc();
  // Clear the application view
  gc.Clear();
  // Draw a rectangle round the edge of the application view.
  TRect drawRect=Rect();
  drawRect.Shrink(0,0);
  gc.SetPenSize(TSize(1,1));
  gc.DrawRect(drawRect);
  TInt ascent= 20;
  gc.UseFont(iFont);
  gc.DrawText(*iCLPText,drawRect,ascent, CGraphicsContext::ECenter,0);
  gc.DrawText(*iCLPQuery,drawRect,50, CGraphicsContext::ELeft,10);
 
  TRect rect=aRect;
  rect.iTl+=TPoint(1,1);
}
