/*******************************************************************************
 **
 ** Licensed Materials - Property of IBM
 **
 ** (C) COPYRIGHT International Business Machines Corp. 2000, 2001
 ** All Rights Reserved.
 **
 ******************************************************************************/

#ifndef __CLPVIEW_H__
#define __CLPVIEW_H__

#include <coecobs.h>
#include <coecntrl.h>
#include <eikcmbut.h>               // for buttons
#include <eiktbar.h>
#include <eikchlst.h>               // for Choice list
#include <eikclb.h>                 // for column list box
#include <eikcmbox.h>               // for Combo Box
#include <eiktxlbx.h>               // for text list boxes builds on eiklbx.h
#include <eiklbx.h>                 // for list boxes
#include <eikclbd.h>                // CColumnListBoxData
#include <bamdesca.h>
#include <eikimage.h>
#include <eikenv.h>
#include <eikprogi.h>
#include <s32std.h>
#include <eikedwin.h>
#include <w32std.h>
#include <eikappui.h>               // menus, etc.
#include <basched.h>

#include <eiktxlbm.h>               //for CTextListBoxModel

#include <SampleCLP.rsg>            // Resource file 
#include <SampleCLP.mbg>            // header for color/or mono multi-bitmaps

#include "CLP.hrh"

#include "CLPsplashview.h"
#include "CLPbitmap.h"
#include "CLPcommon.h"

extern "C"
  {
#include "SampleCLP.h"
#include "SampleCLPmsg.h"
  }

/******************************************************************************/
/** Declaration of CONSTANT                                                  **/
/******************************************************************************/

const TInt KComboTextLimit=256;

/******************************************************************************/
/** Declaration of CLASSES                                                   **/
/******************************************************************************/

class   CCLPAppUi;

/******************************************************************************
  * CCLPClpView Class
  *
  * @param  tableView                 
  * @param  qTableViewFetchStatement  
  *
  * @return qRc                      
  */

class CCLPView : public CCoeControl,
                 public MCoeControlObserver,
                 public MCoeControlBrushContext                        
  {
public:
  ~CCLPView();
  void ConstructL(const TRect& aRect,
                  CLPState*    clpstate,
                  CCLPAppUi* aAppUi);

  IMPORT_C TInt ExecuteLD(TInt aResourceId);
  TInt GetStateScheduleListBoxModelL();
  void AddToListBoxL(const TDesC& aText);
  void SetFontL(const TFontSpec& aFontSpec);
  void HandleCommandL(TInt aCommand);
  void PrintMsg(SQLTCHAR* msg);
  TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);


  private: // from CCoeControl
  void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);
  void Draw(const TRect& aRect) const;

  private: // framework
  void CreateBitMapsL();
  TInt CountComponentControls() const;
  void CreateCombosL();
  void CreateTxLstBoxL(const TRect& aRect,const CFont* afont);
  void LoadBitmapL(CFbsBitmap* aBitMap,
                   const TDesC& aPathAndFile,
                   TInt aId,TBool aShareIfLoaded);
  void ExecuteQueryL();
  
  CCoeControl* ComponentControl(TInt aIndex) const;

  private:
  TBuf<256>           KsqlString;
  CCoeControl*        iAppView;
  TViewType           iViewType;
 
  const CFont*        iFont;
  TFontSpec           iFontSpec;

  TDesC*              iCLPText;
  TDesC*              iCLPQuery;
  TDesC*              iCLPQueryHint;
  TDesC*              iCLPResults;
  TDesC*              iMSG_EXECUTED_OK_ALERT; 
  TDesC*              iERR_OCCURED;
  TDesC*              iMSG_CLP_MSG_ROW_INTICATOR;
  TDesC*              iMSG_CLP_MSG_ROW_COUNT;
  TDesC*              iMSG_CLP_MSG_STMT_OK;
  TDesC*              iMSG_CLP_MSG_STMT_ERROR;
  TDesC*              iMSG_CLP_MSG_STMT_NULLSTATE_ERROR;

  TDesC*              iMSG_CLP_MSG_STMT_RST;

  CFbsBitmap*         iClpInUseBmpi;
  CFbsBitmap*         iClpInUseBmpm;
  CFbsBitmap*         iDbInUseBmpi;
  CFbsBitmap*         iDbInUseBmpm;
  
  CLPState*           iclpstate;
   
public:
  CCLPAppUi*          iAppUi;
  CCoeControl*        iFocusControl;  //rupesh
  CEikTextListBox*    iTxLstBx;
  CEikComboBox*       iCombo[4];

  };

#endif  // __CLPVIEW_H__
