/*****************************************************************************
 *
 * Source File Name = DB2eJavaCLP.java
 *
 * (C) COPYRIGHT International Business Machines Corp. 2001
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Operating System = Java Sample Application
 *
 * This sample application shows how to interactively execute queries
 * against DB2 Everyplace database.
 *
 *
 * Description:
 *
 * Sample Java program for DB2 Everyplace JDBC driver for PalmOS. Class file
 * compliled using this Java file has been confirmed to run on the following
 * Platforms: WinNT, EPOC, WinCE, PalmOS
 *
 *****************************************************************************
 */

package com.ibm.db2e.app;

import java.sql.*;
import java.util.*;
import java.io.*;

public class DB2eJavaCLP
{
  public static final int    UNDEFINED_STMT = -1;
  public static final int    INVALID_STMT   = -2;
  public static final int    QUERY_TYPE     =  1;
  public static final int    UPDATE_TYPE    =  2;
  public static final int    MAX_QUERY_LEN  = 40;
  public static final String DB_DRIVER      = "com.ibm.db2e.jdbc.DB2eDriver";
  public static final String DB_URL         = "jdbc:db2e:mysample";
  
  private ResultSet          rSet   = null;
  private ResultSetMetaData  rMD    = null;
  private Connection         con    = null;
  private Statement          stmt   = null;
  
  
  /*******************************************************************************
   *
   * Function   : public DB2eJavaCLP()
   *
   * Description: Constructor
   *
   *******************************************************************************
   */

  public DB2eJavaCLP()
  {
	try
	  {
		Class.forName(DB_DRIVER).newInstance();
		con = DriverManager.getConnection(DB_URL);
		stmt = con.createStatement();
	  }		
	catch (Exception e)
	  {
		System.out.println(e.toString());		
	  }
  }
  

  
  /*******************************************************************************
   *
   * Function   : public String getQuery()
   *
   * Description: Function for reading queries from standard input. This function
   *              does not work with VAME J9 CLDC Class libraries.
   *
   *******************************************************************************
   */

  public String getQuery()
  {
	try
	  {
		byte [] data = new byte[MAX_QUERY_LEN];
		String query;

		// backspace may not work for some systems
		int i = 0, c = -1;
		while (i < data.length) {
			c = java.lang.System.in.read();
			if (c == -1 || c == '\n')
				break;
			else
			 	data[i++] = (byte)c;
		}

		query = new String(data, 0, i);
		return (query.trim());
	  } 
	catch (IOException e)
	  {
		System.out.println(e.toString());	
		return (null);
	  }
  }
  

  /*******************************************************************************
   *
   * Function   : public void stop()
   *
   * Description: No Object.finalize() method available so we have to explicitly 
   *              clean up.
   *              
   *******************************************************************************
   */

	public void stop()
	{
		try
			{
				if (stmt != null) {
					stmt.close();
				}
  	
				if (con != null) {
					con.close();
				}
			}
		catch (SQLException sqlEx)
			{
				while(sqlEx != null)
					{
						System.out.println("SQLERROR: \n" + sqlEx.getErrorCode() + 
										   ", SQLState: " + sqlEx.getSQLState() + 
										   ", Message: " + sqlEx.getMessage() +
										   ", Vendor: " + sqlEx.getErrorCode() );
						sqlEx = sqlEx.getNextException();
					}
			} 
	}




	/*******************************************************************************
   *
   * Function   : public String executeQuery()
   *
   * Description: Wrapper function for executing select queries
   *              
   *
   *******************************************************************************
   */
  
  public boolean executeQuery(String query)
  {
	try
	  {
		rSet = stmt.executeQuery(query);
		rMD = rSet.getMetaData();
		return (true);
	  }
	catch (SQLException sqlEx)
	  {
		while(sqlEx != null){
		  System.out.println("SQLERROR: \n" + sqlEx.getErrorCode() + 
							 ", SQLState: " + sqlEx.getSQLState() + 
							 ", Message: " + sqlEx.getMessage() +
							 ", Vendor: " + sqlEx.getErrorCode() );
		  sqlEx = sqlEx.getNextException();
		}
		return (false);
	  }
	catch (Exception e)
	  {
		System.out.println(e.toString());
		return (false);
	  }
  }



  /*******************************************************************************
   *
   * Function   : public String executeUpdate()
   *
   * Description: Wrapper function for executing non-select sql queries.
   *              
   *
   *******************************************************************************
   */  
  
  public int executeUpdate(String query)
  {
	try
	  {
		return(stmt.executeUpdate(query));
	  }
	catch (SQLException sqlEx)
	  {
		while(sqlEx != null){
		  System.out.println("SQLERROR: " + sqlEx.getErrorCode() + 
							 ", SQLState: " + sqlEx.getSQLState() + 
							 ", Message: " + sqlEx.getMessage() +
							 ", Vendor: " + sqlEx.getErrorCode() );
		  sqlEx = sqlEx.getNextException();
		}
		return (-1);
	  }
	catch (Exception e)
	  {
		System.out.println(e.toString());
		return (-1);
	  }
  }

	

  /*******************************************************************************
   *
   * Function   : public static String format(String in, int len, String pad, int padMode)
   *
   * Description: Format a string before printing them out using System.out(...)
   *              
   *
   *******************************************************************************
   */  

  public static String format(String in, int len, String pad, int padMode)
  {
	int    diff   = 0;
	String result = "";
	
	if (in == null) {
	  in = "";
	}
	
	diff = len - in.length();
	
	if (pad.length() > 1) {
	  pad = pad.substring(0,1);
	}
	
	if (diff < 0) {
	  result = in.substring(0,len);
	} else {
	  if (padMode == -1) {
		for (int i = 0; i < diff; i++) {
		  result += pad;
		}
	  }
	  
	  result += in;
	  
	  if (padMode == 1) {
		for (int i = 0; i < diff; i++) {
		  result += pad;
		}
	  }
	}
	
	return (result);
  }
  


  /*******************************************************************************
   *
   * Function   : public static int stmtType(String query)
   *
   * Description: Determine if the query is a non-select query. Non select queries
   *              need to be executed using executeUpdate(...) function. For some
   *              reason java.sql does not provide a single method for executing
   *              both select and non-select queries.
   *
   *******************************************************************************
   */  

	public static int stmtType(String query)
	{
		int stmtType = DB2eJavaCLP.UNDEFINED_STMT;
		int check = 0;
		
		query = query.toUpperCase();
		
		if (query.indexOf("SELECT",0) != -1 && query.indexOf("FROM",0) != -1) {
			stmtType = DB2eJavaCLP.QUERY_TYPE;
		} else {
			stmtType = DB2eJavaCLP.UPDATE_TYPE;
		}
		
		return (stmtType);
	}
	


  /*******************************************************************************
   *
   * Function   : public void displayResultSet()
   *
   * Description: Display results of a query in tabular format.
   *
   *******************************************************************************
   */  

	public void displayResultSet()
	{
		int    numCols = 0;
		int    colSize = 0;
		int    numRows = 0;
		String colName = null;
		
		
		try
			{	
				System.out.print("\n");
				// get column headers from resultset				
				for (int i = 1; i <= rMD.getColumnCount(); i++) {
					System.out.print(DB2eJavaCLP.format(rMD.getColumnName(i).toUpperCase(), rMD.getColumnDisplaySize(i)," ",1));  
					System.out.print("  ");
				}
				System.out.print("\n");
				
				for (int i = 1; i <= rMD.getColumnCount(); i++)  {
					System.out.print(DB2eJavaCLP.format("-", rMD.getColumnDisplaySize(i), "-" ,1));
					System.out.print("  ");
				}
				System.out.print("\n");
				
				//get and display rows
				while (rSet.next()) {
					for (int i = 1; i <= rMD.getColumnCount(); i++) {
						System.out.print(DB2eJavaCLP.format(rSet.getString(i), rMD.getColumnDisplaySize(i), " ",1));
						System.out.print("  ");
					}
					numRows += 1;
					System.out.print("\n");
				}
				
				System.out.print("\n");
				// print row count
				System.out.println("Number of rows: " + numRows + "\n");
			} 
		catch (SQLException sqlEx)
			{
				while(sqlEx != null)
					{
						System.out.println("SQLERROR: \n" + sqlEx.getErrorCode() + 
										   ", SQLState: " + sqlEx.getSQLState() + 
										   ", Message: " + sqlEx.getMessage() +
										   ", Vendor: " + sqlEx.getErrorCode() );
						sqlEx = sqlEx.getNextException();
					}
			}
	}

	

  /*******************************************************************************
   *
   * Function   : public static void main(String[] args) throws Exception
   *
   * Description: demonstrates use of JDBC driver in querying DB2 Everyplace
   *              database in non-interactive mode.
   *
   *******************************************************************************
   *

	public static void main(String[] args) throws Exception
	{
		String      query    = null;
		DB2eJavaCLP clp      = new DB2eJavaCLP();
						
		// create a table
		clp.executeUpdate("create table x(a int, b timestamp)");
		
		// insert data into table
		clp.executeUpdate("insert into x values(1, current timestamp)");
		clp.executeUpdate("insert into x values(2, current timestamp)");
		
		// excute select query
		clp.executeQuery("select * from x");
		clp.displayResultSet(); // only for select queries
		
		// query = clp.getQuery(); 
		// clp.executeQuery(query);
		// clp.displayResultSet();

		clp.stop(); // IMPORTANT: always call this function before program exits.
	}
  */
	
  /*******************************************************************************
   *
   * Function   : public static void main(String[] args) throws Exception
   *
   * Description: demonstrates use of JDBC driver in querying DB2 Everyplace
   *              database in interactive mode.
   *
   *******************************************************************************
   */  	


	public static void main(String[] args) throws Exception
	{
		String query          = null;
		ResultSetMetaData rMD = null;
		ResultSet         rS  = null;
		//
		int    numCols = 0;
		int    colSize = 0;
		int    numRows = 0;
		String colName = null;
		//
		DB2eJavaCLP clp = new DB2eJavaCLP();
	
		System.out.println("\n\n\n------------------------------------------------------------");
		System.out.println("       DB2 Everyplace Command Line Processor For Java");
		System.out.println("------------------------------------------------------------\n");		
	
		for (;;) 
			{
				// initialize result set row count
				numRows = 0;
				
				// get the query
				System.out.print("CLP:> ");
				System.out.flush();
				query = clp.getQuery();
				
				if (query.toLowerCase().indexOf("quit",0) == 0 || query.toLowerCase().indexOf("exit",0) == 0) {
					break;
				}
				
				switch(clp.stmtType(query))
					{
					case DB2eJavaCLP.UPDATE_TYPE:
						{
							numRows = clp.executeUpdate(query);
							System.out.println("\n" + numRows + " rows updated\n");
							break;
						}
					case DB2eJavaCLP.QUERY_TYPE:
						{
							if (clp.executeQuery(query) == true) {
								clp.displayResultSet();
							}
							break;
						}
					default:
						{
							break;
						}
					}
			}
		clp.stop(); // important: always call this function before the program exits.
	}
}





