/******************************************************************************/
/*   OCO Source Materials.                                                    */
/*                                                                            */
/*   5622-388 (C) Copyright IBM Corp. 2000, 2001.                             */
/*                                                                            */
/******************************************************************************/
/******************************************************************************/
/**                  DB2e CLP Application (Init Code)		                 **/
/******************************************************************************/
/** Functions : This file contains the source code to create and initializes **/
/**             the database for Phonwbook application.				         **/
/**             For simplification, error recovery procedures are minimal.   **/
/******************************************************************************/
/** Revision History                                                         **/
/**                                                                          **/
/** No    Date          By                   Reason For Change               **/
/** --    ----          --                   -----------------               **/
/** 01  Sept 00		 Jing Gu				 Creation						 **/
/** 02  Aug 01          Xin Hu               Fix bug for "connect to" command**/
/******************************************************************************/

#include "Res.h"
#include "resource.h"					//Windows Resource File
#include "DB2eCLPUIData.h"			//User Interface Data
#include "sampleCLP.h"

#if defined(WINCE300)
	#include <aygshell.h>
#endif

#define MAX_BUF 256
#define MAX_LEN 2050	//length of input
#define MAX_OUT_BUF 256
#define MAX_HISTORY_DEPTH	5

typedef struct _POS{				//Structure recording the location and size of a window
	int x, y, cx, cy;
}POS;

/***********************************************************************
 * Functions for this module
 **********************************************************************/
int InitApp(HINSTANCE);
HWND InitInstance(HINSTANCE, LPWSTR, int);
int TermInstance(HINSTANCE, int);
int AppStarted();
int StartApp();
void StopApp();

int ErrFatal(unsigned long line, TCHAR* ERR_MSG);

LRESULT CALLBACK FrameWndProc(HWND, UINT, WPARAM, LPARAM);
HWND CreateEmptyMenuBar(HWND hWnd, UINT menuID);

LRESULT CreateFrameWnd(HWND, UINT, WPARAM, LPARAM);
LRESULT DestroyFrameWnd(HWND, UINT, WPARAM, LPARAM);

void InitPos(POS *pos, int x, int y, int cx, int cy);

/***********************************************************************
 * Global variables for this module
 **********************************************************************/

HWND hwndMain;	//handle of main window
HINSTANCE hInst;		//Instance Handle
HBRUSH hStaticBrush; 
int originalWidth = 0;
HANDLE	hwndMB;

#if defined(WINCE300)
	SHACTIVATEINFO	s_sai;    // SIP handling structure 
#endif

const TCHAR szAppName[] = TEXT(APPLICATION_NAME);
const TCHAR TITLE[] = TEXT(APPLICATION_TITLE);
const TCHAR VERSION[] = TEXT(DIALOG_VERSION_MESSAGE);
const TCHAR VERSIONTITLE[] = TEXT(DIALOG_VERSION_TITLE);

SQLHENV henv;
SQLHDBC  hdbc;
SQLHSTMT hstmt;
CLPState* clpstate;

SQLTCHAR server[] = TEXT("\\");
SQLTCHAR uid[] = TEXT("tester");
SQLTCHAR pwd[] = TEXT("tester");

/*----------------------------------------------------------------------------*/

TCHAR tstr_title[] = TEXT(DIALOG_ERROR_TITLE);

TCHAR str_buffer[MAX_BUF];	//string buffer
TCHAR input_buffer[MAX_LEN];
int count = 0;	//history count

TCHAR ERROR_MSG_INIT[] = TEXT(ERR_INIT);
TCHAR ERROR_MSG_INPUT[] = TEXT(ERR_INPUT_FAILURE);

/************************************************************************
 *Attributes of Controls - 
 *	Open window, OK button, Name label, Close Button, Version Button
 ************************************************************************/
TCHAR *szFrameWndsClass[] = {TEXT("STATIC"), TEXT("BUTTON"), TEXT("STATIC"), TEXT("COMBOBOX"), 
						     TEXT("BUTTON"), TEXT("BUTTON"), TEXT("BUTTON")};
TCHAR *szFrameWndsName[] = {TEXT(DIALOG_COPYRIGHT_MESSAGE), TEXT(DIALOG_BUTTON_OK), 
							TEXT(APPLICATION_TITLE), TEXT(""), 
							TEXT(BUTTON_EXEC), TEXT(BUTTON_VERSION), TEXT(BUTTON_CLOSE)};
DWORD FrameWndsStyle[] = {SS_CENTER, BS_DEFPUSHBUTTON, SS_CENTER, 
						  WS_BORDER|CBS_DROPDOWN|WS_VSCROLL|CBS_AUTOHSCROLL, 
						  BS_PUSHBUTTON,
						  BS_PUSHBUTTON, BS_PUSHBUTTON};
/**-----------------------------------------------------------------------------
   Purposes  : Main entry point of the application.
   On entry  :                                                              
     ...
   On return :                                                              
     ...Nothing
   On error  :                                                              
     ...Nothing
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	int rc = 0;
	HWND hwndFrame;

	if (AppStarted())			//If Process Existing
		return 0;

	rc = InitApp(hInstance);
	if (rc)	return rc;

	rc = StartApp();  
	if (rc) return rc;


	hwndFrame = InitInstance(hInstance, lpCmdLine, nCmdShow);
	if (hwndFrame == 0)
		return 0x10;

	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return TermInstance(hInstance, msg.wParam);
}/*End of WinMain*/

/**-----------------------------------------------------------------------------
   Purposes  : Check if a previous process is running
   Input:                                                              
     ...None
   Output:                                                              
     ...0 = Successfully executed
     ...!0 = Error detected
   On error  :                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
BOOL AppStarted()
{
	HWND hWnd;

	hWnd = FindWindow(szAppName, TITLE);	
	if (hWnd){
		SetForegroundWindow ((HWND) (((DWORD)hWnd) | 0x01));    
		return TRUE;
	} 
	
	return FALSE;

}/*End of AppStated*/

/**-----------------------------------------------------------------------------
   Purposes  : Register windows class
   Input:                                                              
     ...None
   Output:                                                              
     ...0 = Successfully executed
     ...!0 = Error detected
   On error  :                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
int InitApp(HINSTANCE hInstance)
{
	WNDCLASS wc;
    INITCOMMONCONTROLSEX icex;

	wc.style		= 0;
	wc.lpfnWndProc	= FrameWndProc;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= hInstance;
	wc.hIcon		= NULL;
	wc.hCursor		= NULL;
	wc.hbrBackground= (HBRUSH)GetSysColorBrush(COLOR_WINDOW);
	wc.lpszMenuName	= NULL;
	wc.lpszClassName= szAppName;

	if (RegisterClass(&wc) == 0)
		return 0;

	icex.dwSize = sizeof (INITCOMMONCONTROLSEX);
    icex.dwICC = ICC_LISTVIEW_CLASSES;
    InitCommonControlsEx (&icex);

	return 0;
}/* End of InitApp*/

/**-----------------------------------------------------------------------------
   Purposes  : Create main window
   Input:                                                              
     ...None
   Output:                                                              
     ...!0 = Successfully executed
     ...0 = Error detected
   On error  :                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
HWND InitInstance(HINSTANCE hInstance, LPWSTR lpCmdLine, int nCmdShow)
{
	HWND hWnd;

	hInst = hInstance;

	hWnd = CreateWindow(szAppName,
						szAppName,
						WS_VISIBLE,
						CW_USEDEFAULT,
						CW_USEDEFAULT,
						CW_USEDEFAULT,
						CW_USEDEFAULT,
						NULL,
						NULL,
						hInstance,
						NULL);

	if (!IsWindow(hWnd))
		return NULL;

	hwndMain = hWnd;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	
	return hWnd;
}/*End of InitInstance*/

/**-----------------------------------------------------------------------------
   Purposes  : Terminate
   Input:                                                              
     ...None
   Output: 
     ...
   On error  :                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
int TermInstance(HINSTANCE hInstance, int nDefRC)
{
	return nDefRC;
}/*End of TermInstance*/


/**-----------------------------------------------------------------------------
   Purposes  : Initialization
   Input: 
     ...None
   Output:
     ...0 = Successfully executed
     ...!0 = Error detected
   On error	:
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
int StartApp()
{

	SQLRETURN rc;

	if (!(clpstate = malloc(sizeof(CLPState))))
      return -1;

    // Allocate and initialize CLPState structure
    rc = CLPStartEngineOnPath(clpstate, server, uid, pwd);
    if (rc)
      {
		   ErrFatal(__LINE__, ERROR_MSG_INIT);
       return (rc);
      }
    else
      {
       clpstate->mode   = CLP_INTERACT;
       clpstate->fpIN   = 0;
       clpstate->fpOUT  = 0;
      }
  
	return 0;

}/*End of StartApp*/

/**-----------------------------------------------------------------------------
   Purposes  : Clean up resource
   Input:                                                              
     ...None
   Output:                                                              
     ...None
   On error  :                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
void StopApp(void)
{
  CLPStopEngine(clpstate);
  free(clpstate);
  return;
}/*End of StopApp*/

/**-----------------------------------------------------------------------------
   Purposes  : Management of the window
   Input:                                                              
     ...
   Output:                                                              
     ...
   On error:                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/

LRESULT CALLBACK FrameWndProc(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
	int wmId;
	int i = 0;

	switch (wMsg)
	{
	case WM_CREATE:
		CreateFrameWnd(hWnd, wMsg, wParam, lParam);	
#if defined(WINCE300)
		s_sai.cbSize = sizeof(SHACTIVATEINFO); 
		hwndMB = CreateEmptyMenuBar(hWnd, IDC_EMPTYMENUBAR);

		SipShowIM(SIPF_OFF);
#endif
		return 0;

	case WM_CTLCOLORSTATIC:
		return (hStaticBrush);

	case WM_COMMAND:
		wmId = LOWORD(wParam);

		switch (wmId)
		{
		case IDC_CLOSEBTN:
			DestroyFrameWnd(hWnd, wMsg, wParam, lParam);
			return 0;

		case IDC_VERSIONBTN:
			MessageBox(hWnd, VERSION, VERSIONTITLE, MB_OK|MB_ICONINFORMATION);
			return 0;

		case IDC_OKBTN:
			DestroyWindow(GetDlgItem(hWnd, IDC_COPYRIGHTTEXT));
			DestroyWindow(GetDlgItem(hWnd, IDC_OKBTN));

			for (i = IDC_TITLETEXT; i <= IDC_RESULTLISTVIEW; i++)
			{
				ShowWindow(GetDlgItem(hWnd, i), SW_SHOW);
			}

			SetFocus(GetDlgItem(hWnd, IDC_CLPCOMBO));

#if defined(WINCE300)
			SipShowIM(SIPF_ON);
#endif
			return 0;

		case IDC_EXECBTN:
			{
				SQLRETURN rc;

				HWND hwndLV = GetDlgItem(hWnd, IDC_RESULTLISTVIEW);
				HWND hwndCombo = GetDlgItem(hWnd, IDC_CLPCOMBO);
				int cw1, cw2;

				GetWindowText(hwndCombo, input_buffer, MAX_LEN);	//get input into input_buffer
				ComboBox_InsertString(hwndCombo, 0, input_buffer);
				if (++count > MAX_HISTORY_DEPTH){
					ComboBox_DeleteString(hwndCombo, --count);
				}


				ListView_DeleteAllItems(hwndLV);
				ListView_SetColumnWidth(hwndLV, 0, originalWidth);
				_tcscpy(clpstate->cmd, input_buffer);        
				rc = callable_CLP(clpstate, hwndLV); 
				if (rc == CLP_QUIT)
				{
					PostQuitMessage(0);
					return 0;
				}

				//put database operations here.
				//for each returned line, copy it to output_buffer

				//adjust width
				cw1 = ListView_GetColumnWidth(hwndLV, 0);
				ListView_SetColumnWidth(hwndLV, 0, LVSCW_AUTOSIZE);
				cw2 = ListView_GetColumnWidth(hwndLV, 0);
				ListView_SetColumnWidth(hwndLV, 0, cw1>cw2?cw1:cw2);

				return 0;
			}
			
		}

		return DefWindowProc(hWnd, wMsg, wParam, lParam);

#if defined (WINCE300)

	case WM_ACTIVATE:
	    return SHHandleWMActivate(hWnd, wParam, lParam, &s_sai, 0);

	case WM_SETTINGCHANGE:
			return SHHandleWMSettingChange(hWnd, wParam, lParam, &s_sai);

	case WM_SIZE:
		{
			int x, y, cx, cy;
			POS versionPos, closePos, lvPos;

			x	= 0;
			y	= 0;
			cx	= (int)(short)LOWORD(lParam);
			cy	= (int)(short)HIWORD(lParam);

			InitPos(&versionPos, x + 5, y + cy - 28, 2*cx/5-10, 23);
			InitPos(&closePos, x + 2*cx/5, y + cy - 28, 2*cx/5-10, 23);
			InitPos(&lvPos, x + 5, y + 101, cx - 10, cy - 134);

			SetWindowPos(GetDlgItem(hWnd, IDC_VERSIONBTN), 0, versionPos.x, versionPos.y, versionPos.cx, versionPos.cy, 0);
			SetWindowPos(GetDlgItem(hWnd, IDC_CLOSEBTN), 0, closePos.x, closePos.y, closePos.cx, closePos.cy, 0);
			SetWindowPos(GetDlgItem(hWnd, IDC_RESULTLISTVIEW), 0, lvPos.x, lvPos.y, lvPos.cx, lvPos.cy, 0);

			UpdateWindow(hWnd);
			return;
		}
#endif

	case WM_DESTROY:
		if (hStaticBrush)
		{
			DeleteObject(hStaticBrush);
		}

		if (hwndMB){
			CommandBar_Destroy(hwndMB);
		}

		PostQuitMessage(0);
		break;
	}

	return DefWindowProc(hWnd, wMsg, wParam, lParam);
}/*End of FrameWndProc*/

#if defined(WINCE300)

//*****************************************************************************
/**
 * Create a empty menu bar for the with SIP connected
 *
 * @param hWnd Handle of the main window
 * @param menuID Identity of menu
 *
 * @return Handle of the menubar
 */
HWND CreateEmptyMenuBar(HWND hWnd, UINT menuID)
{
	SHMENUBARINFO mbi;

	memset(&mbi, 0, sizeof(SHMENUBARINFO));
	mbi.cbSize     = sizeof(SHMENUBARINFO);
	mbi.hwndParent = hWnd;
	mbi.dwFlags    = SHCMBF_EMPTYBAR;
	mbi.nToolBarId = menuID;
	mbi.hInstRes   = hInst;
	mbi.nBmpId     = 0;
	mbi.cBmpImages = 0;

	if (!SHCreateMenuBar(&mbi))
		return NULL;
		
	return mbi.hwndMB;
}

#endif
/**-----------------------------------------------------------------------------
   Purposes  : Initialize a position and size
   Input:                                                              
     ...
   Output:                                                              
     ...
   On error:                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
void InitPos(POS *pos, int x, int y, int cx, int cy)
{
	pos->x = x;
	pos->y = y;
	pos->cx = cx;
	pos->cy = cy;

	return;
}/*End of InitPos*/

/**-----------------------------------------------------------------------------
   Purposes  : Create windows
   Input:                                                              
     ...
   Output:                                                              
     ...0 = Successfully executed
     ...!0 = Error detected
   On error:                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
LRESULT CreateFrameWnd(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hwndChild;
	int x, y, cx, cy;
	RECT rect;
	POS Pos[7];
	int i = 0;
	HICON hIcon;

	hStaticBrush = GetSysColorBrush(COLOR_WINDOW);

	GetClientRect(hWnd, &rect);
	x	= rect.left;
	y	= rect.top;
	cx	= rect.right - x;
	cy	= rect.bottom - y;

	InitPos(&Pos[0], x + cx/6, y + cy/4, 2*cx/3, cy/2);	//copyright text
	InitPos(&Pos[1], x + 3*cx/10 +5, y + 3*cy/4 + 5, 2*cx/5-10, 23);//ok button
	InitPos(&Pos[2], x + 5, y + 5, cx - 10, 18);//title text
	InitPos(&Pos[3], x + 5, y + 28, cx - 10, cy/2);//input combo
	InitPos(&Pos[4], x + 5, y + 53, 2*cx/5-10, 23);	//execute button
	InitPos(&Pos[5], x + 5, y + cy - 28, 2*cx/5-10, 23);//version button
	InitPos(&Pos[6], x + 2*cx/5, y + cy - 28, 2*cx/5-10, 23);//close button

	hIcon = (HICON)SendMessage(hWnd, WM_GETICON, 0, 0);
	if (hIcon == 0)
	{
		hIcon = LoadImage (hInst, MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, 32, 32, 0);
		SendMessage(hWnd, WM_SETICON, FALSE, (LPARAM)hIcon);
	}
	
	for (i = 0; i <= (IDC_CLOSEBTN - IDC_COPYRIGHTTEXT); i++)
	{
		hwndChild = CreateWindow(szFrameWndsClass[i], 
								 szFrameWndsName[i],
								 FrameWndsStyle[i]|WS_VISIBLE|WS_CHILD, 
								 Pos[i].x, Pos[i].y, Pos[i].cx, Pos[i].cy,
								 hWnd, 
								 (HMENU)(IDC_COPYRIGHTTEXT + i),
								 hInst, 
								 NULL);

		if (!IsWindow(hwndChild))
		{
			DestroyWindow(hWnd);
			return 0;
		}

		ShowWindow(hwndChild, SW_HIDE);
	}

	hwndChild = CreateWindowEx(WS_EX_CLIENTEDGE, WC_LISTVIEW, TEXT(""),
							   LVS_REPORT | LVS_NOCOLUMNHEADER | WS_VISIBLE | WS_CHILD | WS_BORDER | WS_VSCROLL,
							   x + 5, y + 81, cx - 10, cy - 114,
							   hWnd, (HMENU)IDC_RESULTLISTVIEW,
							   hInst, NULL);

	if(!IsWindow(hwndChild))
	{
		DestroyWindow(hWnd);
		return 0;
	}
	AddColumn(hwndChild, TEXT(""), 0, cx-12);
	originalWidth = cx-12;
	ShowWindow(hwndChild, SW_HIDE);

	ShowWindow(GetDlgItem(hWnd, IDC_COPYRIGHTTEXT), SW_SHOW);
	ShowWindow(GetDlgItem(hWnd, IDC_OKBTN), SW_SHOW);

	return 0;
}/*End of CreateFrameWnd*/

/**-----------------------------------------------------------------------------
   Purposes  : Destroy windows
   Input:                                                              
     ...
   Output:                                                              
     ...
   On error:                                                              
     ...Nothing                                                             
------------------------------------------------------------------------------*/
/**-----------------------------------------------------------------------------
--------------------------------------------------------------------------------
   Algorithm :                                                              
     1-/                                              
   Data Structure Usage :                                                   
     1-/                                                                
------------------------------------------------------------------------------*/
LRESULT DestroyFrameWnd(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
	DestroyWindow(hWnd);
	return 0;
}/*End of DestroyFrameWnd*/

/*----------------------------------------------------------------------------
Purpose:	Error Report
Input:
Output:
On error:

Algorithm:
Data Structure Usage:
/*----------------------------------------------------------------------------*/
int ErrFatal(unsigned long line, TCHAR* ERR_MSG)
{
	SQLRETURN rc;
	TCHAR sqlstate[8];
	long native_err;
	TCHAR error_msg[80];
	short msglen;

	rc = SQLGetDiagRec(SQL_HANDLE_STMT,hstmt,1,sqlstate, &native_err, error_msg,80,&msglen);
	
	_tcscpy(str_buffer, ERR_MSG);
	if (rc == 0)
	{
		MessageBox(hwndMain,str_buffer,tstr_title,MB_OK|MB_ICONERROR);
	}
	return(-1);
} /* End of ErrFatal() */

